@@ -34,6 +34,7 @@ import metator.validation as mtv
 from docopt import docopt
 from metator.log import logger
 from os.path import exists, dirname, join
+from scipy.sparse import save_npz
 
 
 class AbstractCommand:
@@ -144,7 +145,7 @@ class Network(AbstractCommand):
         # Enable file logging
         now = time.strftime("%Y%m%d%H%M%S")
         log_file = join(
-            self.args["--outdir"], ("metator_network" + now + ".log")
+            self.args["--outdir"], ("metator_network_" + now + ".log")
         )
         mtl.set_file_handler(log_file)
 
@@ -332,7 +333,7 @@ class Partition(AbstractCommand):
         # Enable file logging
         now = time.strftime("%Y%m%d%H%M%S")
         log_file = join(
-            self.args["--outdir"], ("metator_partition" + now + ".log")
+            self.args["--outdir"], ("metator_partition_" + now + ".log")
         )
         mtl.set_file_handler(log_file)
 
@@ -349,7 +350,7 @@ class Partition(AbstractCommand):
             raise ValueError
 
         # Partition the network
-        mtp.partition(
+        clustering_matrix, contigs_data_file = mtp.partition(
             self.args["--algorithm"],
             self.args["--assembly"],
             self.args["--contigs"],
@@ -364,6 +365,12 @@ class Partition(AbstractCommand):
             threads,
         )
 
+        # Save the clustering matrix
+        clustering_matrix_file = join(
+            self.args["--outdir"], "clustering_matrix_partition.txt"
+        )
+        save_npz(clustering_matrix_file, clustering_matrix)
+
         # Delete the temporary folder
         if not self.args["--no-clean-up"]:
             shutil.rmtree(temp_directory)
@@ -466,7 +473,7 @@ class Validation(AbstractCommand):
         # Enable file logging
         now = time.strftime("%Y%m%d%H%M%S")
         log_file = join(
-            self.args["--outdir"], ("metator_validation" + now + ".log")
+            self.args["--outdir"], ("metator_validation_" + now + ".log")
         )
         mtl.set_file_handler(log_file)
 
@@ -489,7 +496,7 @@ class Validation(AbstractCommand):
             logger.error('algorithm should be either "louvain" or "leiden"')
             raise ValueError
 
-        mtv.recursive_decontamination(
+        clustering_matrix = mtv.recursive_decontamination(
             self.args["--algorithm"],
             self.args["--assembly"],
             self.args["--contigs"],
@@ -506,6 +513,12 @@ class Validation(AbstractCommand):
             threads,
         )
 
+        # Save the clustering matrix
+        clustering_matrix_file = join(
+            self.args["--outdir"], "clustering_matrix_recursive.txt"
+        )
+        save_npz(clustering_matrix_file, clustering_matrix)
+
         # Delete the temporary folder
         if not self.args["--no-clean-up"]:
             shutil.rmtree(temp_directory)
@@ -624,7 +637,7 @@ class Pipeline(AbstractCommand):
 
         # Enable file logging
         now = time.strftime("%Y%m%d%H%M%S")
-        log_file = join(self.args["--outdir"], ("metator" + now + ".log"))
+        log_file = join(self.args["--outdir"], ("metator_" + now + ".log"))
         mtl.set_file_handler(log_file)
 
         # Define variable
@@ -800,7 +813,7 @@ class Pipeline(AbstractCommand):
             network_file = self.args["--network"]
 
         # Partition the network
-        contigs_data_file = mtp.partition(
+        clustering_matrix, contigs_data_file = mtp.partition(
             self.args["--algorithm"],
             fasta,
             contigs_data_file,
@@ -824,7 +837,7 @@ class Pipeline(AbstractCommand):
 
         # Launch validation if desired.
         if not self.args["--skip-validation"]:
-            mtv.recursive_decontamination(
+            clustering_matrix_recursive = mtv.recursive_decontamination(
                 self.args["--algorithm"],
                 fasta,
                 contigs_data_file,
@@ -841,12 +854,23 @@ class Pipeline(AbstractCommand):
                 threads,
             )
 
+            # Make the sum with the partiton clustering matrix
+            clustering_matrix = (
+                (clustering_matrix + clustering_matrix_recursive) / 2
+            ).tocoo()
+
             # Remove contig_data_partition file
             contig_data_partition_file = join(
                 self.args["--outdir"], "contig_data_partition.txt"
             )
             os.remove(contig_data_partition_file)
 
+        # Save the clustering matrix
+        clustering_matrix_file = join(
+            self.args["--outdir"], "clustering_matrix.txt"
+        )
+        save_npz(clustering_matrix_file, clustering_matrix)
+
         # Delete the temporary folder.
         if not self.args["--no-clean-up"]:
             shutil.rmtree(temp_directory)
